/* TypeCut Tracker add-on */

#include <Be.h>
#include <AppKit.h>
#include <SupportKit.h>
#include <StorageKit.h>
#include <TrackerAddOn.h>
#include <stdio.h>

// the name of the add_on
#define ADDON_NAME "TypeCut"
#define ADDON_VERS "001"

status_t	read_type			();
void 		error				(const char *message);
void		install_myself		();
void		addon_is_at_home	();
void		myAboutWindow		();
void		error_check			(const char * message, int32 status);
status_t	remove_type			(BMessage *msg);

// Globals,					// for auto-install
app_info 	my_app_info;	// getting ref of app
BNode 		my_app_node;	// getting the node of app

BEntry 		file_entry;		// ref to file
BNode		file_node;		// type of file
attr_info	file_attr_info;	// checking if attribute X exists

status_t	status;
BString		type;

extern "C" void 
process_refs(entry_ref dir_ref, BMessage *msg, void *)
{
	printf("\nTypeCut -- process_refs()\n");

	status = msg->FindRef("refs", &dir_ref);
	error_check("msg->FindRef()", status);

	status = file_entry.SetTo(&dir_ref);
	error_check("file_entry.SetTo()", status);

	file_node.SetTo(&dir_ref);
	error_check("file_node.SetTo()", status);

	// read type
	status	=	read_type();
	if (status != B_OK)
	{
		error_check("read_type() failed: ", status);
		return;
	}
	else
		printf("read_type() successful. :)\n");

	// Copy type to clip_board
	if ( be_clipboard->Lock() )
	{ 
		status	=	be_clipboard->Clear();
		if (status != B_OK)
		{	error_check("be_clipboard->Clear(): ", status); }
		
		BMessage *clipper = be_clipboard->Data();
		if (clipper == NULL)
		{	printf("be_clipboard->Data(): ");
			be_clipboard->Unlock();
			return; }
		
		status	=	clipper->AddString("JonasSundstrom-TypeCutCopyPaste", type.String()); 
		if (status != B_OK)
		{	error_check("clipper->AddString(): ", status); 
			be_clipboard->Unlock();
			return; }

		status	=	be_clipboard->Commit(); 
		if (status != B_OK)
		{	error_check("be_clipboard->Commit(): ", status); }

		be_clipboard->Unlock();
	}
	else
		printf("Could not lock clipboard.\n");

	printf("\n");

	remove_type(msg);
}

int main(int argc, char *argv[], char *arge[]) 
{
	printf("\nTypeCut -- main()\n");

	new BApplication("application/x-vnd.JonasSundstrom-TypeCut-addon");

	// Getting my app_info
	be_app->GetAppInfo(& my_app_info);

	// Setting up a node to read/write my attributes
	status = my_app_node.SetTo (& my_app_info.ref);

	if (status != B_NO_ERROR)
	{ printf("Uh, Hullo?..\n It's so dark I can't see where I am..\n"); return 1; }

	addon_is_at_home();	

	delete be_app;
	return 0;
}

void myAboutWindow()
{
  (new BAlert(0, ADDON_NAME " " ADDON_VERS " Tracker add-on\n\n"
              "Compiled by Jonas Sundström.\n"
			  "Copyleft " B_UTF8_COPYRIGHT " 1999 Mirilla Software.\n\n"
              "Install-Me functionality by Raphael Moll.\n\n"
			  "It should be harmless, but you never know..\n\n"
			  "Feedback goes to:\n"
			  "s96j1su@csd.uu.se\n"
              "http://www.csd.uu.se/~s96j1su/",
              "Interesting.."))->Go(); 
}

status_t
read_type()
{
	ssize_t		read_bytes = 0;
	char my_buffer[B_FILE_NAME_LENGTH];

	status = file_node.GetAttrInfo("BEOS:TYPE", &file_attr_info);
		error_check("read_type() : file_node.GetAttrInfo : ", status);

	if (status == B_ENTRY_NOT_FOUND)
	{	printf("Couldn't find TYPE-attribute of that file.\n"); return B_ERROR;	}	
	else if (status != B_NO_ERROR)
	{	printf("Darn.. Somethings wrong the TYPE-attribute of that file.\n"); return B_ERROR; }

	read_bytes = file_node.ReadAttr("BEOS:TYPE", B_STRING_TYPE, 0, my_buffer, 256);

	if (read_bytes > 0)
	{
		type.Truncate(0);				// I had some problems to Assign the char * to the BStrin
		type.Append(my_buffer, read_bytes);	// so this is a temporary solution
		printf("Storing type in BString type.\n");		// Oh, the basics can be so annoying sometimes
	}
	else
		return B_ERROR;
	
	error_check("read_type():", read_bytes);

	printf("type : ");
	printf(type.String());
	printf("\n");
	printf("read_bytes: %d", read_bytes);
	printf("\n");
	printf("buffer length: %d", strlen(my_buffer));
	printf("\n");

	return B_OK;
}

void install_myself(void)
/*
	This function is a slightly adapted version of installOurself() from the
	Tracker Add-On ZipMe, written by R'alf <mailto:raphael.moll@capway.com>
	Thanks !

	Copy *this* binary file into /boot/home/config/add-ons/Tracker/,
	making sure the directories exists and can be created.
*/
{
BEntry my_app_entry;	// No need to manipulate my_app_node
BPath	idealPath;
BDirectory addonsDir;
BDirectory trackerDir;

	status = my_app_entry.SetTo( & my_app_info.ref );
	if (status != B_NO_ERROR) goto bail_out_error;

	// The ideal directory is B_USER_ADDONS_DIRECTORY+"./Tracker"
	// create it if doesn't exists

	status = find_directory(B_USER_ADDONS_DIRECTORY, &idealPath, true);
	if (status != B_NO_ERROR && status != B_FILE_EXISTS) goto bail_out_error;

	if (addonsDir.SetTo(idealPath.Path()) != B_NO_ERROR) goto bail_out_error;
	status = addonsDir.CreateDirectory("Tracker", &trackerDir);
	if (status == B_FILE_EXISTS) status = trackerDir.SetTo(&addonsDir, "Tracker");
	if (status != B_NO_ERROR) goto bail_out_error;
	if (trackerDir.InitCheck() != B_NO_ERROR) goto bail_out_error;

	// move executable
	status = my_app_entry.MoveTo(&trackerDir, NULL, true);
	if (status != B_NO_ERROR && status != B_FILE_EXISTS) goto bail_out_error;

	return;

	bail_out_error:
	printf( "Help! Something's wrong. I can't move myself to \n"
			"/boot/home/config/add-ons/Tracker/");
}


void addon_is_at_home()
{
	// Is Add-On really at home ??
	BEntry		my_app_entry;
	BPath		my_app_path;
	status_t	status;

	BString		path_without_name;

	status = my_app_entry.SetTo (& my_app_info.ref, true);
	
	error_check("addon_is_at_home():my_app_entry.SetTo()", status);

	status = my_app_path.SetTo(& my_app_entry);

	error_check("addon_is_at_home():my_app_path.SetTo()", status);

	path_without_name = my_app_path.Path();
	path_without_name.Truncate( strlen(my_app_path.Path()) - strlen(my_app_path.Leaf()) );

	printf(path_without_name.String());
	printf("\n");

	if (path_without_name == "/boot/home/config/add-ons/Tracker/")
	{
		printf("Add-On is at home. Let's get going with the files! \n");
		return;
	}

	// else, ask the user if add-on can move itself.

	BAlert *box = new BAlert(ADDON_NAME,
							 "Hi! I'm " ADDON_NAME " " ADDON_VERS " !\n\n"
							 "I'd like to live in\n"
							 "/boot/home/config/add-ons/Tracker/\n\n"
							 "I could try to move myself, if you like..",
							 "Don't move", "Move", NULL,
							 B_WIDTH_FROM_WIDEST);
	
	if (box->Go() == 1) install_myself();
}

void error_check(const char * message, int32 status)
{
	printf(message);
		if (status == B_OK)					printf("B_OK \n");
		if (status == B_ERROR)				printf("B_ERROR \n");
		if (status == B_ENTRY_NOT_FOUND)	printf("B_ENTRY_NOT_FOUND \n");
		if (status == B_FILE_ERROR)			printf("B_FILE_ERROR \n");
		if (status == B_NOT_ALLOWED)		printf("B_NOT_ALLOWED \n");
		if (status == B_DEVICE_FULL)		printf("B_DEVICE_FULL \n");
		if (status == B_NO_MEMORY)			printf("B_NO_MEMORY \n");
		if (status == B_BAD_VALUE)			printf("B_BAD_VALUE \n");
		if (status == B_BUSY)				printf("B_BUSY \n");
		if (status == B_NO_INIT)			printf("B_NO_INIT \n");
		if (status == B_PERMISSION_DENIED)	printf("B_PERMISSION_DENIED \n");
		if (status == B_FILE_EXISTS)		printf("B_FILE_EXISTS \n");
		if (status == B_BAD_INDEX)			printf("B_BAD_INDEX \n");
		if (status == B_BAD_TYPE)			printf("B_BAD_TYPE \n");
		if (status == B_NAME_NOT_FOUND)		printf("B_NAME_NOT_FOUND \n");
}

status_t
remove_type(BMessage *msg)
{
	printf("remove_type() \n");
	
	uint32 some_type;
	int32 count; 
	entry_ref ref;
	BEntry entry;
	BNode node;
	
	msg->GetInfo("refs", &some_type, &count); 
	if ( some_type != B_REF_TYPE )
	{
		printf("BMessage did not contain refs.\n");
		return;
	}
	
	for (long i=count;  i>=0;  i--)
	{ 
		if ((msg->FindRef("refs", i, &ref) == B_OK)
			&&	(entry.SetTo(&ref, true) == B_OK)
			&&  entry.IsFile()
			&&	node.SetTo(&entry) == B_OK)
		{
			printf("Removing TYPE of file.\n");

		    status = node.RemoveAttr("BEOS:TYPE");

			if (status != B_OK)
			{
				error_check("Could not remove filetype.", status);
				return status;
			}
		}
	}
	return B_OK;
}

