/* TypePaste Add-on */

#include <Be.h>
#include <AppKit.h>
#include <SupportKit.h>
#include <StorageKit.h>
#include <TrackerAddOn.h>
#include <stdio.h>

// the name of the add_on
#define ADDON_NAME "TypePaste"
#define ADDON_VERS "001"

status_t	write_type			(BMessage *msg);
void 		error				(const char * message);
void		install_myself		();
void		addon_is_at_home	();
void		myAboutWindow		();
void		error_check			(const char * message, int32 status);

// Globals,					// for auto-install
app_info 	my_app_info;	// getting ref of app
BNode 		my_app_node;	// getting the node of app

entry_ref	file_ref;		// ref to file
BNode		file_node;		// type of file
attr_info	file_attr_info;	// checking if attribute X exists

status_t	status;
BString		type;

extern "C" void 
process_refs(entry_ref dir_ref, BMessage *msg, void *)
{
	printf("\nTypePaste -- process_refs()\n");

//	status = msg->FindRef("refs", &dir_ref);
//	error_check("msg->FindRef()", status);

	file_ref = dir_ref;		// safe ?

	file_node.SetTo(&file_ref);
	error_check("file_node.SetTo() ", status);

	// Get type from clip_board
	if ( be_clipboard->Lock() )
	{ 
		printf("Trying to get a BMess pointer to clipboard...\n");

		BMessage *clipper = be_clipboard->Data();

		printf("Ok, passed that clause.\n");

		clipper->PrintToStream();

		if (clipper == NULL)
		{	printf("BMessage clipper is NULL. :(");
			be_clipboard->Unlock();
			 return; }
		else
			printf("clipper is not NULL. Good.\n");
		
		char * text;

		status	=	clipper->FindString("JonasSundstrom-TypeCutCopyPaste", &text);
		if (status == B_OK)
		{ 
			printf("Ok, got the string from clipper..\n");
			type = text;

			printf("type: ");
			printf(type.String());
			printf("\n");
		}
		else
		{	error_check("Could not find filetype in clipboard: ", status);
			be_clipboard->Unlock();
			return; }
		printf("Passed the FindString clause.\n");
		be_clipboard->Unlock();
		printf("Passed the Unlock clause.\n");
	}
	else
	{
		printf("Could not lock clipboard.\n");
	}

	// set types of files
	write_type(msg);

	printf("\n");
}

int main(int argc, char *argv[], char *arge[]) 
{
	printf("\nTypePaste -- main()\n");

	new BApplication("application/x-vnd.JonasSundstrom-TypePaste-addon");

	// Getting my app_info
	be_app->GetAppInfo(& my_app_info);

	// Setting up a node to read/write my attributes
	status = my_app_node.SetTo (& my_app_info.ref);

	if (status != B_NO_ERROR)
	{ printf("Uh, Hullo?..\n It's so dark I can't see where I am.."); return(1); }

	addon_is_at_home();	

	delete be_app;

	return 0;
}

void myAboutWindow()
{
  (new BAlert(0, ADDON_NAME " " ADDON_VERS " Tracker add-on\n\n"
              "Compiled by Jonas Sundström.\n"
			  "Copyleft " B_UTF8_COPYRIGHT " 1999 Mirilla Software.\n\n"
              "Install-Me functionality by Raphael Moll.\n\n"
			  "It should be harmless, but you never know..\n\n"
			  "Feedback goes to:\n"
			  "s96j1su@csd.uu.se\n"
              "http://www.csd.uu.se/~s96j1su/",
              "Interesting.."))->Go(); 
}

status_t
write_type(BMessage *msg)
{
	printf("write_type() \n");
	
	uint32 some_type;
	int32 count; 
	entry_ref ref;
	BEntry entry;
	BNode node;
	
	msg->GetInfo("refs", &some_type, &count); 
	if ( some_type != B_REF_TYPE )
	{
		printf("BMessage did not contain refs.\n");
		return;
	}
	
	for (long i=count;  i>=0;  i--)
	{ 
		if ((msg->FindRef("refs", i, &ref) == B_OK)
			&&	(entry.SetTo(&ref, true) == B_OK)
			&&  entry.IsFile()
			&&	node.SetTo(&entry) == B_OK)
		{
			printf("Adding TYPE: ");
			printf(type.String());
			printf(" to file.\n");

			// add BEOS:TYPE attribute

			ssize_t		written_bytes = 0;

		    written_bytes = node.WriteAttr("BEOS:TYPE", B_STRING_TYPE, 0,
								type.String(), type.Length()+1);

			if (written_bytes != type.Length()+1)
			{
				printf("Writing the BEOS:TYPE attribute: \n Written_bytes != Length+1.\n");
				return -1;
			}
	
			printf("\n");
			printf("BEOS:TYPE = ");
			printf(type.String());
			printf(": ");

			printf(type.String());
			printf("\t");
			printf("%d", written_bytes);
			printf("\n");
		}
	}

	return B_OK;
}

void install_myself(void)
/*
	This function is a slightly adapted version of installOurself() from the
	Tracker Add-On ZipMe, written by R'alf <mailto:raphael.moll@capway.com>
	Thanks !

	Copy *this* binary file into /boot/home/config/add-ons/Tracker/,
	making sure the directories exists and can be created.
*/
{
BEntry my_app_entry;	// No need to manipulate my_app_node
BPath	idealPath;
BDirectory addonsDir;
BDirectory trackerDir;

	status = my_app_entry.SetTo( & my_app_info.ref );
	if (status != B_NO_ERROR) goto bail_out_error;

	// The ideal directory is B_USER_ADDONS_DIRECTORY+"./Tracker"
	// create it if doesn't exists

	status = find_directory(B_USER_ADDONS_DIRECTORY, &idealPath, true);
	if (status != B_NO_ERROR && status != B_FILE_EXISTS) goto bail_out_error;

	if (addonsDir.SetTo(idealPath.Path()) != B_NO_ERROR) goto bail_out_error;
	status = addonsDir.CreateDirectory("Tracker", &trackerDir);
	if (status == B_FILE_EXISTS) status = trackerDir.SetTo(&addonsDir, "Tracker");
	if (status != B_NO_ERROR) goto bail_out_error;
	if (trackerDir.InitCheck() != B_NO_ERROR) goto bail_out_error;

	// move executable
	status = my_app_entry.MoveTo(&trackerDir, NULL, true);
	if (status != B_NO_ERROR && status != B_FILE_EXISTS) goto bail_out_error;

	return;

	bail_out_error:
	printf( "Help! Something's wrong. I can't move myself to \n"
			"/boot/home/config/add-ons/Tracker/");
}


void addon_is_at_home()
{
	// Is Add-On really at home ??
	BEntry		my_app_entry;
	BPath		my_app_path;
	status_t	status;

	BString		path_without_name;

	status = my_app_entry.SetTo (& my_app_info.ref, true);
	
	error_check("addon_is_at_home():my_app_entry.SetTo()", status);

	status = my_app_path.SetTo(& my_app_entry);

	error_check("addon_is_at_home():my_app_path.SetTo()", status);

	path_without_name = my_app_path.Path();
	path_without_name.Truncate( strlen(my_app_path.Path()) - strlen(my_app_path.Leaf()) );

	printf(path_without_name.String());
	printf("\n");

	if (path_without_name == "/boot/home/config/add-ons/Tracker/")
	{
		printf("Add-On is at home. Let's get going with the files! \n");
		return;
	}

	// else, ask the user if add-on can move itself.

	BAlert *box = new BAlert(ADDON_NAME,
							 "Hi! I'm " ADDON_NAME " " ADDON_VERS " !\n\n"
							 "I'd like to live in\n"
							 "/boot/home/config/add-ons/Tracker/\n\n"
							 "I could try to move myself, if you like..",
							 "Don't move", "Move", NULL,
							 B_WIDTH_FROM_WIDEST);
	
	if (box->Go() == 1) install_myself();
}

void error_check(const char * message, int32 status)
{
	printf(message);
		if (status == B_OK)					printf("B_OK \n");
		if (status == B_ERROR)				printf("B_ERROR \n");
		if (status == B_ENTRY_NOT_FOUND)	printf("B_ENTRY_NOT_FOUND \n");
		if (status == B_FILE_ERROR)			printf("B_FILE_ERROR \n");
		if (status == B_NOT_ALLOWED)		printf("B_NOT_ALLOWED \n");
		if (status == B_DEVICE_FULL)		printf("B_DEVICE_FULL \n");
		if (status == B_NO_MEMORY)			printf("B_NO_MEMORY \n");
		if (status == B_BAD_VALUE)			printf("B_BAD_VALUE \n");
		if (status == B_BUSY)				printf("B_BUSY \n");
		if (status == B_NO_INIT)			printf("B_NO_INIT \n");
		if (status == B_PERMISSION_DENIED)	printf("B_PERMISSION_DENIED \n");
		if (status == B_FILE_EXISTS)		printf("B_FILE_EXISTS \n");
		if (status == B_BAD_INDEX)			printf("B_BAD_INDEX \n");
		if (status == B_BAD_TYPE)			printf("B_BAD_TYPE \n");
		if (status == B_NAME_NOT_FOUND)		printf("B_NAME_NOT_FOUND \n");
}

